@@ -32,8 +32,6 @@
 			NWDSDefineAttr, NWDSRemoveAttrDef.
 */
 
-#include "config.h"
-
 #include <ncp/nwcalls.h>
 #include "nwnet_i.h"
 
@@ -139,7 +137,7 @@ NWDSCCODE NWDSReadClassDef(
 }
 
 NWDSCCODE NWDSGetClassDefCount(
-		NWDSContextHandle	ctx,
+	UNUSED(	NWDSContextHandle	ctx),
 		Buf_T*			buf,
 		NWObjectCount*		classDefCount
 ) {
@@ -210,7 +208,7 @@ NWDSCCODE NWDSGetClassDef(
 }
 
 NWDSCCODE NWDSGetClassItemCount(
-		NWDSContextHandle	ctx,
+	UNUSED(	NWDSContextHandle	ctx),
 		Buf_T*			buf,
 		NWObjectCount*		itemCount
 ) {
@@ -291,7 +289,7 @@ NWDSCCODE NWDSPutClassItem(
 }
 	
 NWDSCCODE NWDSBeginClassItem(
-		NWDSContextHandle	ctx,
+	UNUSED(	NWDSContextHandle	ctx),
 		Buf_T*			buf
 ) {
 	NWDSCCODE dserr;
@@ -791,14 +789,14 @@ NWDSCCODE NWDSRemoveAttrDef(
 static NWDSCCODE __NWDSSyncSchemaV0(
 		NWCONN_HANDLE		conn,
 		nuint32			flags,
-		nuint32a		time
+		nuint32a		timev
 ) {
 	NW_FRAGMENT rq_frag[1];
 	nuint8 rq_b[12];
 	
 	DSET_LH(rq_b, 0, 0);	/* version */
 	DSET_LH(rq_b, 4, flags);
-	DSET_LH(rq_b, 8, time);
+	DSET_LH(rq_b, 8, timev);
 
 	rq_frag[0].fragAddr.ro = rq_b;
 	rq_frag[0].fragSize = 12;
