@@ -32,6 +32,19 @@
 	1.01  2000, January 16		Petr Vandrovec <vandrove@vc.cvut.cz>
 		Changes for 32bit uids.
 
+	1.02  2001, February 24		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added broadcast state information.
+		Added global_fd code.
+	
+	1.03  2001, March 4		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added conn_ring code.
+
+	1.04  2001, June 2		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Moved shuffle() declaration here from ndscrypt.h.
+
+	1.05  2001, July 16		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added put_req_size_unsigned prototype.
+
  */
 
 #ifndef _NCPLIB_I_H
@@ -48,6 +61,14 @@ extern "C" {
 #include "private/libncp-atomic.h"
 #include "private/ncp_fs.h"
 
+#include <string.h>
+
+#define UNUSED(x) x __attribute__((unused))
+
+#define ncp_const_cast(x) ((void*)(unsigned long)(x))
+
+#define NWCC_BCAST_PERMIT_UNKNOWN	-9876	/* something unused */
+
 struct ncp_conn {
 
 	enum connect_state is_connected;
@@ -83,17 +104,23 @@ struct ncp_conn {
 	u_int8_t sequence;
 	ncpt_atomic_t use_count;	/* it was completion code */
 	int conn_status;
-	int reply_size;
 
 	/* Fields used to setup ncp requests */
 	char *current_point;
 	int has_subfunction;
 	int verbose;
 	size_t ncp_reply_size;
+	
+	void *private_key;
+	size_t private_key_len;
 
 	int lock;
 
 	char packet[NCP_PACKET_SIZE];
+	char *ncp_reply;
+	char *ncp_reply_buffer;
+	size_t ncp_reply_alloc;
+
 	/* Field used to make packet signatures */
 	int sign_wanted;
 	int sign_active;
@@ -106,7 +133,15 @@ struct ncp_conn {
 		struct {
 			unsigned int major, minor, revision;
 		} version;
+		int ncp64bit;
 	} serverInfo;
+	
+	enum NET_ADDRESS_TYPE nt;
+	
+	int bcast_state;
+	int global_fd;
+	
+	struct list_head conn_ring;
 };
 
 /* saving pointer somewhere */
@@ -178,6 +213,11 @@ static inline void ncp_add_dword_hl(struct ncp_conn *conn, dword x) {
 	conn->current_point += 4;
 }
 
+static inline void ncp_add_qword_hl(struct ncp_conn *conn, u_int64_t x) {
+	QSET_HL(conn->current_point, 0, x);
+	conn->current_point += 8;
+}
+
 DEFINE_ASSERT_FUNCTION(ncp_add_byte,byte)
 DEFINE_ASSERT_FUNCTION(ncp_add_word_lh,word)
 DEFINE_ASSERT_FUNCTION(ncp_add_word_hl,word)
@@ -201,7 +241,7 @@ static inline int ncp_add_seek(struct ncp_conn *conn, int pos) {
 void ncp_add_pstring(struct ncp_conn *conn, const char *s);
 
 static inline char* ncp_reply_data(struct ncp_conn *conn, int offset) {
-	return &(conn->packet[sizeof(struct ncp_reply_header) + offset]);
+	return conn->ncp_reply + sizeof(struct ncp_reply_header) + offset;
 }
 
 static inline byte ncp_reply_byte(struct ncp_conn *conn, int offset) {
@@ -224,24 +264,43 @@ static inline dword ncp_reply_dword_lh(struct ncp_conn *conn, int offset) {
 	return DVAL_LH(ncp_reply_data(conn, offset), 0);
 }
 
+static inline u_int64_t ncp_reply_qword_lh(struct ncp_conn *conn, int offset) {
+	return QVAL_LH(ncp_reply_data(conn, offset), 0);
+}
+
+static inline u_int32_t cpu_to_le32(u_int32_t val) {
+	return DVAL_LH(&val, 0);
+}
+
+static inline u_int32_t cpu_to_be32(u_int32_t val) {
+	return htonl(val);
+}
+
 static inline unsigned int min(unsigned int a, unsigned int b) {
 	return (a < b) ? a : b;
 }
 
-static inline void ConvertToNWfromDWORD(u_int32_t sfd, u_int8_t nwhandle[6]) {
+static inline void ConvertToNWfromDWORD(u_int32_t sfd, char nwhandle[6]) {
 	DSET_LH(nwhandle, 2, sfd);
 	WSET_LH(nwhandle, 0, sfd+1);
 }
 
+static inline u_int32_t ConvertToDWORDfromNW(const char nwhandle[6]) {
+	return DVAL_LH(nwhandle, 2);
+}
+
 void ncp_init_request(struct ncp_conn *conn);
 void ncp_init_request_s(struct ncp_conn *conn, int subfunction);
-int ncp_attach_by_addr(const struct sockaddr* target, struct ncp_conn** conn);
 
 long ncp_sign_start(struct ncp_conn *conn, const char *sign_root);
 
+void shuffle(const unsigned char *objid, const unsigned char *pwd, int buflen, unsigned char *out);
+
 NWCCODE ncp_set_private_key(struct ncp_conn *conn, const void* pk, size_t pk_len);
 NWCCODE ncp_get_private_key(struct ncp_conn *conn, void* pk, size_t* pk_len);
 
+NWCCODE ncp_next_conn(struct ncp_conn *conn, struct ncp_conn **nextConn);
+
 #ifdef SIGNATURES
 static inline int ncp_get_sign_wanted(struct ncp_conn *conn) {
 	return conn->sign_wanted;
@@ -255,6 +314,20 @@ static inline int ncp_get_sign_active(struct ncp_conn *conn) {
 #define ncp_get_sign_active(X) (0)
 #endif
 
+extern ncpt_mutex_t nds_ring_lock;
+
+NWCCODE ncp_put_req_size_unsigned(void *buffer, size_t reqlen, unsigned long value);
+
+NWCCODE x_recvfrom(int sock, void *buf, int len, unsigned int flags,
+	       struct sockaddr *sender, socklen_t* addrlen, int timeout,
+	       size_t *rlen);
+
+static inline NWCCODE x_recv(int sock, void *buf, int len, unsigned int flags, int timeout, size_t *rlen) {
+	return x_recvfrom(sock, buf, len, flags, NULL, 0, timeout, rlen);
+}
+
+NWCCODE __NWReadFileServerInfo(struct ncp_conn* conn);
+
 #ifdef __cplusplus
 }
 #endif
