@@ -27,25 +27,34 @@
 #ifndef _NCPSIGN_H
 #define _NCPSIGN_H
 
-#include <ncp/ncplib.h>
+#include "ncplib_i.h"
 
 #ifdef SIGNATURES
-
-void sign_init(const char *logindata, char *sign_root);
-void sign_packet(struct ncp_conn *conn, int *size);
-
+void sign_init(const unsigned char *logindata, unsigned char *sign_root);
+int sign_verify_reply(struct ncp_conn *conn, const void *data, size_t size, u_int32_t totalsize, const void *sign_buf);
 #else
-
-static inline void sign_init(const char *logindata, char *sign_root) {
+static inline void sign_init(const unsigned char *logindata, unsigned char *sign_root) {
 	(void)logindata;
 	(void)sign_root;
 }
-
-static inline void sign_packet(struct ncp_conn *conn, int *size) {
+int sign_verify_reply(struct ncp_conn *conn, const void *data, size_t size, u_int32_t totalsize, const void *sign_buf) {
+	return 1;
 	(void)conn;
+	(void)data;
 	(void)size;
+	(void)totalsize;
+	(void)sign_buf;
+}
+#endif
+
+void __sign_packet(struct ncp_conn *conn, const void *data, size_t size, u_int32_t totalsize, void *sign_buf);
+
+static inline size_t sign_packet(struct ncp_conn *conn, const void *data, size_t size, u_int32_t totalsize, void *sign_buf) {
+	if (ncp_get_sign_active(conn)) {
+		__sign_packet(conn, data, size, totalsize, sign_buf);
+		return 8;
+	}
+	return 0;
 }
 
 #endif
-
-#endif
