@@ -31,12 +31,25 @@
 		
 	1.02  2000, April 23		Petr Vandrovec <vandrove@vc.cvut.cz>
 		Moved nds_login to ds/setkeys.c, added __NWDSGetPrivateKey.
+
+	1.03  2000, July 6		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added DCV_DEREF_ALIASES into authentication functions.
 		
+	1.04  2000, July 10		Gerhard Lausser <GerhardLausser@KirchPayTV.de>
+	        Fixed strcpy_wc call in nds_login_auth.
+
+	1.05  2001, March 9		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added NWIsDSServerW. For now ISO-8859-1 only.
+
+	1.06  2001, May 31		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Use correct parameter (nuint32) to GetLE32().
+
 */
 
 #define RANDBUF  /* if defined: read random data once from /dev/urandom */
 /*#define ERR_MSG*/ /* if defined: show error messages in nds_login_auth */
 /*#define DEBUG_PRINT*/
+/*#define FIND_ISR */ /* if defined: show reasons for -330 invalid response */
 
 #include "config.h"
 
@@ -65,11 +78,18 @@ typedef unsigned char boolean;
 #ifdef ERR_MSG
 #include <stdio.h>
 
-#include <libintl.h>
+#include "private/libintl.h"
 #define _(X) dgettext(PACKAGE, (X))
 #define N_(X) (X)
 #endif
 
+#ifdef FIND_ISR
+#include <stdio.h>
+#define ISRPrint(X...) do { fprintf(stderr, __FILE__ ":%d: ", __LINE__); fprintf(stderr, X); } while (0)
+#else
+#define ISRPrint(X...) do {} while (0)
+#endif
+
 static void strcpy_cw(wchar_t *w, const char* s) {
 	while ((*w++ = *(const nuint8*)s++) != 0);
 }
@@ -97,6 +117,18 @@ NWDSCCODE NWIsDSServer(NWCONN_HANDLE conn, char* treename) {
 	return 1;	
 }
 
+NWDSCCODE NWIsDSServerW(NWCONN_HANDLE conn, wchar_t* treename) {
+	char treechar[MAX_TREE_NAME_CHARS + 1];
+	NWDSCCODE err;
+	
+	err = NWIsDSServer(conn, treechar);
+	if (err && treename) {
+		char* treesrc = treechar;
+		do { } while ((*treename++ = *treesrc++ & 0xFF) != 0);
+	}
+	return err;
+}
+
 static NWDSCCODE nds_read_pk(NWDSContextHandle ctx, const wchar_t* objname, 
 		Octet_String_T** post) {
 	char attrname[256];
@@ -122,15 +154,20 @@ static NWDSCCODE nds_read_pk(NWDSContextHandle ctx, const wchar_t* objname,
 	err = NWDSGetAttrCount(ctx, &replybuf, &cnt);
 	if (err)
 		return err;
-	if (cnt != 1)
+	if (cnt != 1) {
+		ISRPrint("Attr count != 1 (%d)\n", cnt);
 		return ERR_INVALID_SERVER_RESPONSE;
+	}
 	err = NWDSGetAttrName(ctx, &replybuf, (NWDSChar*)name, &valcnt, &synt);
 	if (err)
 		return err;
 	if ((synt != SYN_OCTET_STRING) ||
 	    (wcscmp(name, L"Public Key")) ||
-	    (valcnt < 1))
+	    (valcnt < 1)) {
+	    	ISRPrint("Attr type wrong (synt=%d, valcnt=%d)\n",
+	    		synt, valcnt);
 	    	return ERR_INVALID_SERVER_RESPONSE;
+	}
 	err = NWDSComputeAttrValSize(ctx, &replybuf, SYN_OCTET_STRING, &size);
 	if (err)
 		return err;
@@ -293,66 +330,73 @@ static void dumpkey(const nuint8 *key, size_t keylen) {
 #endif
 
 static int checkkey(const char *key) { /* 0 - wrong key, != 0 - key ok */
-    char temp[8];
-    char *keyptr, *p;
-    size_t keylen;
+	char temp[8];
+	char *keyptr, *p;
+	size_t keylen;
     
-    if ((initkey(key, &keyptr, &keylen)) && 
-     (findchunk(keyptr, keylen, "MA", &p))) {
+	if ((initkey(key, &keyptr, &keylen)) && 
+	    (findchunk(keyptr, keylen, "MA", &p))) {
 #ifdef DEBUG_PRINT
-     	dumpkey(keyptr, keylen);
+		mpkey(keyptr, keylen);
 #endif
-        nwhash1init(temp, 8);
-        nwhash1(temp, 8, key + 10, WVAL_LH(key, 8) - 20);
-        return (!memcmp(p, temp, 8));
-    } else
-        return 0;
-    
+		nwhash1init(temp, 8);
+		nwhash1(temp, 8, key + 10, WVAL_LH(key, 8) - 20);
+		return (!memcmp(p, temp, 8));
+	}
+	return 0;
 }
 
 static ncpt_mutex_t mpilock = NCPT_MUTEX_INITIALIZER;
 
 static long modexpkey(const char *s_key, char *buf, char *outbuf, int bufsize) {
-    char *s_keyptr;
-    size_t s_keylen;
-    int i, nbits, nblocksize;
-    int err = -1;
-    unitptr nmod, nexp, nin, nout;
-    char *p;
+	char *s_keyptr;
+	size_t s_keylen;
+	int i, nbits, nblocksize;
+	int err = -1;
+	unitptr nmod, nexp, nin, nout;
+	char *p;
 
-    nmod = nexp = nin = nout = NULL;
+	nmod = nexp = nin = nout = NULL;
     
-    if (!initkey(s_key, &s_keyptr, &s_keylen))
-        return ERR_INVALID_SERVER_RESPONSE;
-    i = findchunk(s_keyptr, s_keylen, "NN", &p);
-    if (!p)
-        return ERR_INVALID_SERVER_RESPONSE;
-    nbits = countbits_l(p, i);
-    nblocksize = ((nbits + 31) & (~31)) >> 3;
-    if (!(nmod = malloc(nblocksize * 4)))
-        return ENOMEM;
-    copyfill(nmod, nblocksize, p, i);
-    i = findchunk(s_keyptr, s_keylen, "EN", &p);
-    err = ERR_INVALID_SERVER_RESPONSE;
-    if (!p) goto end1;
-    nexp = (unitptr)(((char*)nmod) + nblocksize);
-    nin = (unitptr)(((char*)nexp) + nblocksize);
-    nout = (unitptr)(((char*)nin) + nblocksize);
-    copyfill(nexp, nblocksize, p, i);
-    copyfill(nin, nblocksize, buf, bufsize);
-    ncpt_mutex_lock(&mpilock);
-    set_precision(bytes2units(nblocksize));
-    i = mp_modexp(nout, nin, nexp, nmod);
-    ncpt_mutex_unlock(&mpilock);
-    if (i)
-        err = ERR_INVALID_SERVER_RESPONSE;
-    else {
-        copyfill(outbuf, bufsize, nout, nblocksize);
-        err = 0;
-    }
+	if (!initkey(s_key, &s_keyptr, &s_keylen)) {
+    		ISRPrint("Initkey failed\n");
+		return ERR_INVALID_SERVER_RESPONSE;
+	}
+	i = findchunk(s_keyptr, s_keylen, "NN", &p);
+	if (!p) {
+		ISRPrint("NN chunk not found\n");
+		return ERR_INVALID_SERVER_RESPONSE;
+	}
+	nbits = countbits_l(p, i);
+	nblocksize = ((nbits + 31) & (~31)) >> 3;
+	if (!(nmod = malloc(nblocksize * 4)))
+		return ENOMEM;
+	copyfill(nmod, nblocksize, p, i);
+	i = findchunk(s_keyptr, s_keylen, "EN", &p);
+	err = ERR_INVALID_SERVER_RESPONSE;
+	if (!p) {
+		ISRPrint("EN chunk not found\n");
+		goto end1;
+	}
+	nexp = (unitptr)(((char*)nmod) + nblocksize);
+	nin = (unitptr)(((char*)nexp) + nblocksize);
+	nout = (unitptr)(((char*)nin) + nblocksize);
+	copyfill(nexp, nblocksize, p, i);
+	copyfill(nin, nblocksize, buf, bufsize);
+	ncpt_mutex_lock(&mpilock);
+	set_precision(bytes2units(nblocksize));
+	i = mp_modexp(nout, nin, nexp, nmod);
+	ncpt_mutex_unlock(&mpilock);
+	if (i) {
+		ISRPrint("mp_modexp failed\n");
+		err = ERR_INVALID_SERVER_RESPONSE;
+	} else {
+		copyfill(outbuf, bufsize, nout, nblocksize);
+		err = 0;
+	}
 end1:    
-    free(nmod);
-    return err;
+	free(nmod);
+	return err;
 }
 
 /* ctx must be in WCHAR_T mode, without DCV_CANONICALIZE_NAMES */
@@ -371,6 +415,7 @@ static NWDSCCODE get_public_key(NWDSContextHandle ctx, const wchar_t* objname, n
 	if ((ofs > keylen) || (!initkey(keybuf + ofs, &kptr, &klen)) || 
 		(klen + ofs > keylen) || (!checkkey(keybuf + ofs))) {
 		err = ERR_INVALID_SERVER_RESPONSE;
+		ISRPrint("Wrong key\n");
 		goto err_exit;
 	}
 	if (key) {
@@ -391,7 +436,7 @@ static NWDSCCODE __AlignAndEncryptBlockWithSK(
 		const nuint8* hash, 
 		const nuint8* idata, 
 		size_t ilen, 
-		void* odata, 
+		char* odata, 
 		size_t* olen
 ) {
 	size_t	filler;
@@ -429,7 +474,7 @@ NWDSCCODE __NWEncryptWithSK(
 	for (i = 0; i < 10; i++) {
 		nwhash1(hash, 8, hashsrc, hashsrclen);
 	}
-	err = __AlignAndEncryptBlockWithSK(hash, idata, ilen, odata + 12, &len);
+	err = __AlignAndEncryptBlockWithSK(hash, idata, ilen, (char*)odata + 12, &len);
 	if (err)
 		return err;
 	*olen = len + 12;
@@ -569,7 +614,7 @@ static NWDSCCODE NWDXSetKeys(
 	mlock(ndai, tlen);
 	NWDXSetKeysOnConns(dxh, ndai, tlen);
 	if (dxh->authinfo) {
-		size_t tlen = dxh->authinfo->header.total;
+		tlen = dxh->authinfo->header.total;
 		memset(dxh->authinfo, 0, tlen);
 		munlock(dxh->authinfo, tlen);
 		free(dxh->authinfo);
@@ -675,7 +720,7 @@ NWDSCCODE __NWDSGetPrivateKey(
 	size_t bl;
 	NWDSCCODE err;
 	NWDSCCODE grace_err;
-	size_t len;
+	nuint32 len;
 	size_t n2;
 	size_t n3;
 	size_t i;
@@ -723,17 +768,24 @@ NWDSCCODE __NWDSGetPrivateKey(
 		goto err_exit;
 	}
 	err = ERR_INVALID_SERVER_RESPONSE;
-	if (len < 12)
+	if (len < 12) {
+		ISRPrint("Key shorter than 12 bytes (%d)\n", len);
 		goto err_exit;
+	}
 		
 	xprintf("%04X %04X %02X\n", DVAL_LH(p, 0), DVAL_LH(p, 4), WVAL_LH(p, 8));
 
 	if ((DVAL_LH(p, 0) != 0x00000001) ||
-	    (DVAL_LH(p, 4) != 0x00060001))
+	    (DVAL_LH(p, 4) != 0x00060001)) {
+	    	ISRPrint("Invalid key start (%08X %08X %04X)\n",
+	    		 DVAL_LH(p, 0), DVAL_LH(p, 4), WVAL_LH(p, 8));
 	    	goto err_exit;
+	}
 	n3 = WVAL_LH(p, 8);
-	if (len < n3 + 12)
+	if (len < n3 + 12) {
+		ISRPrint("Invalid key len (%d < %d)\n", len, n3 + 12);
 		goto err_exit;
+	}
 	p += 12;
 	
 	nwhash1init(temp, 8);
@@ -747,12 +799,14 @@ NWDSCCODE __NWDSGetPrivateKey(
 	xprintf("G1\n");
 	
 	if (memcmp(temp, p + n3 - 5, 5)) {
+		ISRPrint("Wrong hash\n");
 		goto err_exit;
 	}
 	
 	xprintf("G2\n");
 		
 	if (memcmp(p, tempbuf, 4)) {
+		ISRPrint("Wrong hash #2\n");
 		goto err_exit;
 	}
 	
@@ -767,6 +821,8 @@ NWDSCCODE __NWDSGetPrivateKey(
 
 	if ((DVAL_LH(p, 0) != 0x00000001) ||
 	    (DVAL_LH(p, 4) != 0x00060001)) {
+	    	ISRPrint("Invalid start #2: %08X %08X\n", DVAL_LH(p, 0),
+	    		DVAL_LH(p, 4));
 	    	goto err_exit;
 	}
 		
@@ -783,6 +839,8 @@ NWDSCCODE __NWDSGetPrivateKey(
 
 	if ((DVAL_LH(p, 0) != 1) ||
 	    (WVAL_LH(p, 4) != 2)) {
+	    	ISRPrint("Invalid start #3: %08X %08X\n", DVAL_LH(p, 0),
+	    		DVAL_LH(p, 4));
 	    	goto err_exit;
 	}
 		
@@ -806,6 +864,7 @@ NWDSCCODE __NWDSGetPrivateKey(
 		
 		if (!checkkey(tb)) {
 			free(tb);
+			ISRPrint("Post check failed\n");
 			goto err_exit;
 	        }
 		
@@ -857,33 +916,51 @@ static NWDSCCODE nds_beginauth2(
 	err = ERR_INVALID_SERVER_RESPONSE;
 	p = rpb.curPos;
 	pend = rpb.dataend;
-	if (pend - p < 12)
+	if (pend - p < 12) {
+		ISRPrint("Invalid len: %u < 12\n", pend - p);
 		goto err_exit;
+	}
 	if ((DVAL_LH(p, 0) != 0x00000001) ||
-	    (DVAL_LH(p, 4) != 0x00020009))
+	    (DVAL_LH(p, 4) != 0x00020009)) {
+	    	ISRPrint("Invalid start: %08X %08X\n", DVAL_LH(p, 0),
+	    		DVAL_LH(p, 4));
 	    	goto err_exit;
+	}
 	n3 = DVAL_LH(p, 8);
-	if (n3 < 16)
+	if (n3 < 16) {
+		ISRPrint("Invalid len: %u < 16\n", n3);
 		goto err_exit;
+	}
 	p += 12;
-	if (pend - p < n3)
+	if (pend - p < n3) {
+		ISRPrint("Invalid len: %u < %u\n", pend - p, n3);
 		goto err_exit;
+	}
 	pend = p + n3;
 	k1tl = DVAL_LH(p, 0);
-	if (k1tl < 12)
+	if (k1tl < 12) {
+		ISRPrint("K1Tl %u < 12\n", k1tl);
 		goto err_exit;
+	}
 	p += 4;
 	k1end = p + k1tl;
 	if ((DVAL_LH(p, 0) != 0x00000001) ||
-	    (DVAL_LH(p, 4) != 0x000A0001))
+	    (DVAL_LH(p, 4) != 0x000A0001)) {
+	    	ISRPrint("Invalid start: %08X %08X\n", DVAL_LH(p, 0),
+	    		DVAL_LH(p, 4));
 	    	goto err_exit;
+	}
 	n3a = WVAL_LH(p, 8);
 	p += 12;
-	if (k1end - p < n3a)
+	if (k1end - p < n3a) {
+		ISRPrint("Invalid len: %u < %u\n", k1end - p, n3a);
 		goto err_exit;
+	}
 	n1 = ((countbits_l(p, n3a) + 31) & (~31)) >> 3;
-	if (n1 < 52)
+	if (n1 < 52) {
+		ISRPrint("Too short key: %u < 52\n", n1);
 		goto err_exit;
+	}
 	if (!(n_temp = malloc(n1))) {
 		err = ENOMEM;
 		goto err_exit;
@@ -891,49 +968,63 @@ static NWDSCCODE nds_beginauth2(
 	copyfill(n_temp, n1, p, n3a);
 	p = (void*)(((unsigned long)k1end + 3) & ~3);
 	err = modexpkey(s_key, n_temp, n_temp, n1);
-	if (err)
+	if (err) {
+		ISRPrint("modexpkey failed\n");
 		goto err_exit;
+	}
 	err = ERR_INVALID_SERVER_RESPONSE;
 	nwhash1init(temp, 7);
 	nwhash1(temp + 5, 2, n_temp, 45);
 	nwhash1(temp, 5, n_temp + 1, 39);
-	if (memcmp(temp, n_temp + 40, 7))
+	if (memcmp(temp, n_temp + 40, 7)) {
+		ISRPrint("Invalid hash\n");
 		goto err_exit;
+	}
 	nwhash1init(temp, 8);
 	for (n1 = 10; n1; n1--)
 		nwhash1(temp, 8, n_temp + 1, 28);
 	free(n_temp); n_temp = NULL;
-	if (pend - p < 16)
+	if (pend - p < 16) {
+		ISRPrint("Invalid len: %u < 16\n", pend - p);
     		goto err_exit;
+    	}
 	if ((DVAL_LH(p, 0) != 28) ||
 	    (DVAL_LH(p, 4) != 0x00000001) ||
 	    (DVAL_LH(p, 8) != 0x00060001) ||
-	    (DVAL_LH(p, 12) != 0x00040010))
+	    (DVAL_LH(p, 12) != 0x00040010)) {
+	    	ISRPrint("Invalid start: %u %08X %08X %08X\n", DVAL_LH(p, 0), 
+	    		DVAL_LH(p, 4), DVAL_LH(p, 8), DVAL_LH(p, 12));
 	    	goto err_exit;
+	}
 	p += 16;
-	if (pend - p < 16)
+	if (pend - p < 16) {
+		ISRPrint("Invalid len: %u < 16\n", pend - p);
 		goto err_exit;
+	}
 	nwdecryptblock(temp, p, 16, p);
 	nwhash1init(temp, 5);
 	nwhash1(temp, 5, p, 11);
 	if ((!memcmp(temp, p + 11, 5)) || (!memcmp(p, randno, 4)))
 		err = 0;
+	else
+		ISRPrint("Invalid hash\n");
 err_exit:        
 	if (n_temp) free(n_temp);
 	return err;
 }
 
-static char *allocfillchunk(const char *keyptr, int keylen, const char *chunk,
- int destsize) {
-    char *p, *p2;
-    int i;
-    i = findchunk(keyptr, keylen, chunk, &p);
-    if (!p) 
-        return NULL;
-    if (!(p2 = malloc(destsize)))
-        return NULL;
-    copyfill(p2, destsize, p, i);
-    return p2;    
+static char *allocfillchunk(const char *keyptr, int keylen,
+		const char *chunk, int destsize) {
+	char *p, *p2;
+	int i;
+
+	i = findchunk(keyptr, keylen, chunk, &p);
+	if (!p) 
+		return NULL;
+	if (!(p2 = malloc(destsize)))
+		return NULL;
+	copyfill(p2, destsize, p, i);
+	return p2;    
 }
 
 static NWDSCCODE nds_beginauth(
@@ -962,7 +1053,7 @@ static NWDSCCODE gen_auth_data(
 	char *keyptr;
 	size_t keylen;
 	int i, j;
-	int nbits, nblocksize, nbytes;
+	int nbits, nblocksize, nbytes, nblocksize_nw;
 	unsigned char nmask;
 	unitptr n_mod, n_exp, n_pn, n_qn, n_dp, n_dq, n_cr, n_key, n_temp;
 	unitptr n_key_dp, n_key_dq;
@@ -974,24 +1065,30 @@ static NWDSCCODE gen_auth_data(
 	NWDSCCODE err;
     
 	n_temp = n_mod = n_exp = n_pn = n_qn = n_dp = n_dq = n_cr = n_key = NULL;
-	if (!initkey(u_key, &keyptr, &keylen))
+	if (!initkey(u_key, &keyptr, &keylen)) {
+		ISRPrint("Initkey failed\n");
 		return ERR_INVALID_SERVER_RESPONSE;
+	}
 	i = findchunk(keyptr, keylen, "NN", &p);
-	if (!p)
+	if (!p) {
+		ISRPrint("NN chunk not found\n");
 		return ERR_INVALID_SERVER_RESPONSE;
+	}
 	nbits = countbits_l(p, i);
 	nbytes = (nbits + 7) >> 3;
 	nmask = (unsigned char)(255 >> (8 - (nbits & 7)));
 	/* we really want (x + 31) & ~15... I'm sorry, but Novell thinks that way */
-	nblocksize = ((nbits + 31) & (~15)) >> 3;
+	nblocksize_nw = ((nbits + 31) & (~15)) >> 3;
 
 	ncpt_mutex_lock(&mpilock);
-	set_precision(bytes2units(nblocksize));
+	set_precision(bytes2units(nblocksize_nw));
+	nblocksize = units2bytes(global_precision);
    
 	n_mod = (unitptr)allocfillchunk(keyptr, keylen, "NN", nblocksize);
 	n_exp = (unitptr)allocfillchunk(keyptr, keylen, "EN", nblocksize);
 	if (!initkey(u_priv_key, &keyptr, &keylen)) {
 		err = ERR_INVALID_SERVER_RESPONSE;
+		ISRPrint("initkey failed\n");
 		goto err_exit;
 	}
 	n_pn = (unitptr)allocfillchunk(keyptr, keylen, "PN", nblocksize);
@@ -1037,12 +1134,12 @@ static NWDSCCODE gen_auth_data(
 	randbuf = (unitptr)(((char*)n_key_dq) + nblocksize);
 	memset(randbuf, 0, nblocksize * 3);
 
-	p = NWDSBufPutPtr(outbuf, 12 + nblocksize * 6);
+	p = NWDSBufPutPtr(outbuf, 12 + nblocksize_nw * 6);
 	DSET_LH(p, 0, 0x00000001);
 	DSET_LH(p, 4, 0x00100008);
 	WSET_LH(p, 8, 3);
-	WSET_LH(p, 10, nblocksize * 3);
-	memset(p + 12, 0, nblocksize * 6);
+	WSET_LH(p, 10, nblocksize_nw * 3);
+	memset(p + 12, 0, nblocksize_nw * 6);
 
 	up = randbuf; up2 = (unitptr)(p + 12);
 	for (i = 3; i; i--) {
@@ -1057,9 +1154,9 @@ static NWDSCCODE gen_auth_data(
 		}
 		mp_modexp(up2, up, n_exp, n_mod);
 		((char *)up) += nblocksize; 
-		((char *)up2) += nblocksize;
+		((char *)up2) += nblocksize_nw;
 	}
-	nwhash2block(hashbuf, p+12, nblocksize * 3);
+	nwhash2block(hashbuf, p+12, nblocksize_nw * 3);
 	nwhash2end(hashbuf);
 
 	up = randbuf;
@@ -1069,7 +1166,7 @@ static NWDSCCODE gen_auth_data(
 		stage_modulus(n_mod);
 		mp_modmult(up2, up2, up); 
 		((char *)up) += nblocksize; 
-		((char *)up2) += nblocksize;
+		((char *)up2) += nblocksize_nw;
 	}
 	if (n_temp) {
     		mp_init0(n_temp);
@@ -1135,7 +1232,7 @@ NWDSCCODE NWDSAuthenticateConn(
 	err = NWDSDuplicateContextHandle(inctx, &ctx);
 	if (err)
 		goto err_exit3;
-	ctx->dck.flags = DCV_XLATE_STRINGS | DCV_TYPELESS_NAMES;
+	ctx->dck.flags = DCV_XLATE_STRINGS | DCV_TYPELESS_NAMES | DCV_DEREF_ALIASES;
 	ctx->priv_flags |= DCV_PRIV_AUTHENTICATING;
 	err = NWDSSetContext(ctx, DCK_LOCAL_CHARSET, "WCHAR_T//");
 	if (err)
@@ -1230,20 +1327,22 @@ long nds_login_auth(struct ncp_conn *conn, const char *user,
     err = NWDSCreateContextHandle(&ctx);
     if (err)
     	return err;
-    ctx->dck.flags = DCV_XLATE_STRINGS | DCV_TYPELESS_NAMES;
+    ctx->dck.flags = DCV_XLATE_STRINGS | DCV_TYPELESS_NAMES | DCV_DEREF_ALIASES | DCV_CANONICALIZE_NAMES;
     ctx->priv_flags |= DCV_PRIV_AUTHENTICATING;
+    err = NWDSXlateFromCtx(ctx, user_w, sizeof(user_w), user);
+    if (err)
+	goto err_exit;
     err = NWDSSetContext(ctx, DCK_LOCAL_CHARSET, "WCHAR_T//");
     if (err)
     	goto err_exit;
     NWDSAddConnection(ctx, conn);
-    strcpy_cw(user_w, user);
-    err = nds_login(ctx, (const wchar_t*)user_w, pwd);
+    err = nds_login(ctx, (const NWDSChar*)user_w, pwd);
     if ((err == ERR_NO_SUCH_ENTRY) &&
+	(user_w[0] != '.') &&
         (user_w[wcslen(user_w)-1] != '.')) {
 #ifdef ERR_MSG        
-        strcpy_wc(buf, user_w);
         printf(_("User %s not found in current context.\n"
-                 "Trying server context...\n"), buf);
+                 "Trying server context...\n"), user);
 #endif
 	err = __NWDSGetServerDN(conn, server_name, sizeof(server_name));
 	if (err)
@@ -1258,7 +1357,8 @@ long nds_login_auth(struct ncp_conn *conn, const char *user,
 	memcpy(user_w + wcslen(user_w), 
 		server_name + i,
 		(wcslen(server_name + i) + 1) * sizeof(wchar_t));
-	err = nds_login(ctx, user_w, pwd);
+	ctx->dck.flags &= ~DCV_CANONICALIZE_NAMES;
+	err = nds_login(ctx, (const NWDSChar*)user_w, pwd);
     }
     if (err) {
         if (err != NWE_PASSWORD_EXPIRED) {
