@@ -37,9 +37,14 @@
 	1.01  1999, November 21		Petr Vandrovec <vandrove@vc.cvut.cz>
 		Copy MAC resource fork.
 
- */
+	1.02  2000, June 17		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Recursive copy.
 
-#include "config.h"
+	1.03  2001, August 9		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Copy trustees and other Netware informations on directories.
+		Added -u option to copy different files only.
+
+ */
 
 #include <stdio.h>
 #include <unistd.h>
@@ -52,9 +57,10 @@
 #include <fcntl.h>
 #include <signal.h>
 #include <errno.h>
+#include <dirent.h>
 #include <ncp/nwcalls.h>
 
-#include <libintl.h>
+#include "private/libintl.h"
 #define _(X) gettext(X)
 
 #define dfprintf(X...) fprintf(X)
@@ -63,7 +69,7 @@
  * Globals:
  *
  */
-const char *VersionStr = "1.01";
+const char *VersionStr = "1.02";
 static char *ProgramName;
 
 #define RSRCSUBDIR	".rsrc/"
@@ -72,6 +78,7 @@ static char *ProgramName;
 
 static int optVersion = 0;		/* -V TRUE if just want version */
 static int optVerbose = 0;		/* -v TRUE if want verbose output */
+static int optRecursive = 0;		/* -r TRUE if want recursive copy */
 static int optNice = 0;			/* -n TRUE if we are cooperative (nice) */
 static int optNiceFactorSel = 0;	/* -s TRUE if we selected a nice factor */
 static int optNiceFactor = 10;		/* -s arg, number of 1M blocks to copy 
@@ -85,6 +92,7 @@ static int optTimes = 0;		/* -p too... Hack source... */
 static int optTrustees = 0;		/* -t */
 static int optMAC = 0;			/* -m */
 static int optMACinRsrc = 0;		/* -M */
+static int optOnlyChanged = 0;		/* -u */
 static int BlocksCopied = 0;		/* Number of blocks copied */
 static unsigned int MaxNcopyRetries = 25;	/* Maximum number of times to retry a failed
 				   copy before giving up  */
@@ -103,11 +111,11 @@ static struct sigaction sTermSig;
  *
  */
 static void
-usage(void)
-{
+usage(void) {
 	fprintf(stderr, _("usage: %s [-V]\n"), ProgramName);
 	fprintf(stderr, _("       %s [-vmMnppt] [-s amt] sourcefile destinationfile|directory\n"), ProgramName);
 	fprintf(stderr, _("       %s [-vmMnppt] [-s amt] sourcefile [...] directory\n"), ProgramName);
+	fprintf(stderr, _("       %s [-vmMnppt] [-s amt] -r sourcedir directory\n"), ProgramName);
 }
 
 /****************************************************************************
@@ -117,8 +125,7 @@ usage(void)
  * Return pointer to original string if no "/" in string. (except at end)
  */
 static const char *
-myBaseName(const char *path)
-{
+myBaseName(const char *path) {
 	const char *p;
 
 	for (p = &path[strlen(path)]; p != path; p--)
@@ -159,7 +166,7 @@ handleOptions(const int argc, char *const argv[])
 {
 	int opt;
 
-	while ((opt = getopt(argc, argv, "vVns:ptmM")) != EOF)
+	while ((opt = getopt(argc, argv, "vVns:ptmMru")) != EOF)
 	{
 		switch (opt)
 		{
@@ -205,6 +212,14 @@ handleOptions(const int argc, char *const argv[])
 			optMACinRsrc = 1;
 			break;
 			
+		case 'r':
+			optRecursive = 1;
+			break;
+
+		case 'u':
+			optOnlyChanged = 1;
+			break;
+
 		default:	/* invalid options or options without required arguments */
 			return 1;
 		}
@@ -230,7 +245,7 @@ validateFileArgs(const int argc, char *const argv[])
 		fprintf(stderr, _("%s: No destination specified.\n"), ProgramName);
 		return 1;
 	}
-	if ((argc > 2) && (p = notDir(argv[argc - 1])))
+	if ((optRecursive || argc > 2) && (p = notDir(argv[argc - 1])))
 	{			/* last arg MUST be dir */
 		fprintf(stderr, _("%s: %s: %s\n"), ProgramName, argv[argc - 1], p);
 		return 1;
@@ -238,7 +253,7 @@ validateFileArgs(const int argc, char *const argv[])
 	return 0;
 }
 
-int xlatid(NWCONN_HANDLE scon, NWObjectID sID,
+static int xlatid(NWCONN_HANDLE scon, NWObjectID sID,
 		NWCONN_HANDLE dcon, NWObjectID *dID) {
 	NWCCODE err;
 	char objectName[256];
@@ -411,7 +426,7 @@ static int
 netwareCopyFile(int fdIn, int fdOut,
 		const char *source,
 		const char *destination,
-		mode_t sourceMode)
+		mode_t sourceMode, int dofile)
 {
 	NWCCODE err;
 	struct nw_info_struct2 sfi, dfi;
@@ -438,11 +453,18 @@ netwareCopyFile(int fdIn, int fdOut,
 		scon = NULL;
 		goto srcDetermined;
 	}
-	err = ncp_ns_open_create_entry(scon, NW_NS_DOS, SA_ALL,
-		1, sentry.volume, sentry.dirEnt, NULL, 0, 
-		-1, OC_MODE_OPEN, 0, AR_READ_ONLY, 
-		RIM_ALL,
-		&sfi, sizeof(sfi), NULL, NULL, sfhandle);
+	if (dofile) {
+		err = ncp_ns_open_create_entry(scon, NW_NS_DOS, SA_ALL | 0x48,
+			1, sentry.volume, sentry.dirEnt, NULL, 0, 
+			-1, OC_MODE_OPEN, 0, AR_READ_ONLY, 
+			RIM_ALL,
+			&sfi, sizeof(sfi), NULL, NULL, sfhandle);
+	} else {
+		err = ncp_ns_obtain_entry_info(scon, NW_NS_DOS, SA_ALL | 0x48,
+			1, sentry.volume, sentry.dirEnt, NULL, 0,
+			NW_NS_DOS, RIM_ALL,
+			&sfi, sizeof(sfi));
+	}
 	if (err) {
 		dfprintf(stderr, _("%s: Unable to open source file: %s\n"),
 			ProgramName, strnwerror(err));
@@ -474,19 +496,20 @@ srcDetermined:;
 		dcon = NULL;
 		goto dstDetermined;
 	}
-	err = ncp_ns_open_create_entry(dcon, NW_NS_DOS, SA_ALL,
-		1, dentry.volume, dentry.dirEnt, NULL, 0,
-		-1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
-		0,
-		&dfi, sizeof(dfi), NULL, NULL, dfhandle);
-	if (err) {
-		dfprintf(stderr, _("%s: Unable to open destination file: %s\n"),
-			ProgramName, strnwerror(err));
-		ncp_close(dcon);
-		dcon = NULL;
-		goto dstDetermined;
+	if (dofile) {
+		err = ncp_ns_open_create_entry(dcon, NW_NS_DOS, SA_ALL,
+			1, dentry.volume, dentry.dirEnt, NULL, 0,
+			-1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
+			0,
+			&dfi, sizeof(dfi), NULL, NULL, dfhandle);
+		if (err) {
+			dfprintf(stderr, _("%s: Unable to open destination file: %s\n"),
+				ProgramName, strnwerror(err));
+			ncp_close(dcon);
+			dcon = NULL;
+			goto dstDetermined;
+		}
 	}
-
 dstDetermined:;
 	if (scon && dcon) {	
 		NWCONN_NUM snum, dnum;
@@ -521,171 +544,174 @@ dstDetermined:;
 			}
 		}
 	}
-	/* Set globals in case a signal happens while copying */
-	CurrentConn = dcon;
-	CurrentFile = dfhandle;
+	if (dofile) {
+		/* Set globals in case a signal happens while copying */
+		CurrentConn = dcon;
+		CurrentFile = dfhandle;
 
-	retValue = copyNWNW(_("NetWare copy"),
-			    fdIn, scon, sfhandle, source,
-			    fdOut, dcon, dfhandle, destination);
-
-	if (dcon) {
-		if (ncp_close_file(dcon, dfhandle) != 0)
-		{
-			fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, destination);
-			retValue = 1;
-		}
-		CurrentFile = NULL;
-	}
-	if (scon) {
-		if (ncp_close_file(scon, sfhandle) != 0)
-		{
-			fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, source);
-			if (!retValue)
+		retValue = copyNWNW(_("NetWare copy"),
+				    fdIn, scon, sfhandle, source,
+				    fdOut, dcon, dfhandle, destination);
+		if (dcon) {
+			if (ncp_close_file(dcon, dfhandle) != 0)
+			{
+				fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, destination);
 				retValue = 1;
+			}
+			CurrentFile = NULL;
 		}
-	}
-	if ((optMAC || optMACinRsrc) && !retValue) {
-		NWCONN_HANDLE sMAC = NULL;
-		NWCONN_HANDLE dMAC = NULL;
-		int sFD = -1;
-		int dFD = -1;
-		const char* sourceRsrc = source;
-		const char* destinationRsrc = destination;
-		char sourceB[MAXPATHLEN+1];
-		char destinationB[MAXPATHLEN+1];
+		if (scon) {
+			if (ncp_close_file(scon, sfhandle) != 0)
+			{
+				fprintf(stderr, _("%s: Close failed for %s\n"), ProgramName, source);
+				if (!retValue)
+					retValue = 1;
+			}
+		}
+		if ((optMAC || optMACinRsrc) && !retValue) {
+			NWCONN_HANDLE sMAC = NULL;
+			NWCONN_HANDLE dMAC = NULL;
+			int sFD = -1;
+			int dFD = -1;
+			const char* sourceRsrc = source;
+			const char* destinationRsrc = destination;
+			char sourceB[MAXPATHLEN+1];
+			char destinationB[MAXPATHLEN+1];
 
-		if (scon && optMAC) {
-			struct nw_info_struct2 srcMAC;
-		
-			err = ncp_ns_obtain_entry_info(scon, NW_NS_DOS, SA_ALL,
-				1, sentry.volume, sentry.dirEnt, NULL, 0,
-				NW_NS_MAC, RIM_DIRECTORY,
-				&srcMAC, sizeof(srcMAC));
-			if (err) {
-				/* it is not fatal... it is user problem... */
-				dfprintf(stderr, _("MAC namespace is not supported on source %s: %s\n"), source, strnwerror(err));
-				/* look for .rsrc */
-			} else {
-				err = ncp_ns_open_create_entry(scon, NW_NS_MAC, SA_ALL,
-					1, srcMAC.Directory.volNumber, srcMAC.Directory.dirEntNum, NULL, 0,
-					1, OC_MODE_OPEN, 0, AR_READ_ONLY,
-					0, 
-					NULL, 0, NULL, NULL, sfhandle);
+			if (scon && optMAC) {
+				struct nw_info_struct2 srcMAC;
+
+				err = ncp_ns_obtain_entry_info(scon, NW_NS_DOS, SA_ALL,
+					1, sentry.volume, sentry.dirEnt, NULL, 0,
+					NW_NS_MAC, RIM_DIRECTORY,
+					&srcMAC, sizeof(srcMAC));
 				if (err) {
-					/* NWE_SERVER_FAILURE is returned on NSS volume if there does not exist resource fork yet... */
-					/* ignore silently */
-					if (err != NWE_SERVER_FAILURE) {
-						fprintf(stderr, _("Unable to open MAC resource fork on source %s: %s\n"), source, strnwerror(err));
-						retValue = 1;
+					/* it is not fatal... it is user problem... */
+					dfprintf(stderr, _("MAC namespace is not supported on source %s: %s\n"), source, strnwerror(err));
+					/* look for .rsrc */
+				} else {
+					err = ncp_ns_open_create_entry(scon, NW_NS_MAC, SA_ALL,
+						1, srcMAC.Directory.volNumber, srcMAC.Directory.dirEntNum, NULL, 0,
+						1, OC_MODE_OPEN, 0, AR_READ_ONLY,
+						0, 
+						NULL, 0, NULL, NULL, sfhandle);
+					if (err) {
+						/* NWE_SERVER_FAILURE is returned on NSS volume if there does not exist resource fork yet... */
+						/* ignore silently */
+						if (err != NWE_SERVER_FAILURE) {
+							fprintf(stderr, _("Unable to open MAC resource fork on source %s: %s\n"), source, strnwerror(err));
+							retValue = 1;
+						}
+						/* no MAC... */
+						goto nomac;
 					}
-					/* no MAC... */
-					goto nomac;
+					sMAC = scon;
 				}
-				sMAC = scon;
 			}
-		}
-		if (!sMAC) {
-			const char* p;
-			char* d;
+			if (!sMAC) {
+				const char* p;
+				char* d;
 			
-			if (!optMACinRsrc)
-				goto nomac;
-			
-			p = strrchr(source, '/');
-			if (p)
-				p = p + 1;
-			else
-				p = source;
-			memcpy(sourceB, source, p - source);
-			d = sourceB + (p - source);
-			d = stpcpy(d, RSRCSUBDIR);
-			strcpy(d, p);
-			sFD = open(sourceB, O_RDONLY);
-			if (sFD == -1)
-				goto nomac;
-			sourceRsrc = sourceB;
-		}
-		if (dcon && optMAC) {
-			struct nw_info_struct2 dstMAC;
+				if (!optMACinRsrc)
+					goto nomac;
 
-			err = ncp_ns_obtain_entry_info(dcon, NW_NS_DOS, SA_ALL,
-				1, dentry.volume, dentry.dirEnt, NULL, 0,
-				NW_NS_MAC, RIM_DIRECTORY,
-				&dstMAC, sizeof(dstMAC));
-			if (err) {
-				/* possible information loss... maybe fallback to create .rsrc/name */
-				fprintf(stderr, _("MAC namespace is not supported on destination %s: %s\n"), destination, strnwerror(err));
-				retValue = 1;
-			} else {
-				err = ncp_ns_open_create_entry(dcon, NW_NS_MAC, SA_ALL,
-					1, dstMAC.Directory.volNumber, dstMAC.Directory.dirEntNum, NULL, 0,
-					1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
-					0,
-					NULL, 0, NULL, NULL, dfhandle);
+				p = strrchr(source, '/');
+				if (p)
+					p = p + 1;
+				else
+					p = source;
+				memcpy(sourceB, source, p - source);
+				d = sourceB + (p - source);
+				d = stpcpy(d, RSRCSUBDIR);
+				strcpy(d, p);
+				sFD = open(sourceB, O_RDONLY);
+				if (sFD == -1)
+					goto nomac;
+				sourceRsrc = sourceB;
+			}
+			if (dcon && optMAC) {
+				struct nw_info_struct2 dstMAC;
+
+				err = ncp_ns_obtain_entry_info(dcon, NW_NS_DOS, SA_ALL,
+					1, dentry.volume, dentry.dirEnt, NULL, 0,
+					NW_NS_MAC, RIM_DIRECTORY,
+					&dstMAC, sizeof(dstMAC));
 				if (err) {
-					/* it is fatal... */
-					fprintf(stderr, _("Unable to create MAC resource fork on destination %s: %s\n"), destination, strnwerror(err));
+					/* possible information loss... maybe fallback to create .rsrc/name */
+					fprintf(stderr, _("MAC namespace is not supported on destination %s: %s\n"), destination, strnwerror(err));
 					retValue = 1;
+				} else {
+					err = ncp_ns_open_create_entry(dcon, NW_NS_MAC, SA_ALL,
+						1, dstMAC.Directory.volNumber, dstMAC.Directory.dirEntNum, NULL, 0,
+						1, OC_MODE_OPEN, 0, AR_WRITE_ONLY,
+						0,
+						NULL, 0, NULL, NULL, dfhandle);
+					if (err) {
+						/* it is fatal... */
+						fprintf(stderr, _("Unable to create MAC resource fork on destination %s: %s\n"), destination, strnwerror(err));
+						retValue = 1;
+						goto nomac;
+					}
+					dMAC = dcon;
+				}
+			}
+			if (!dMAC) {
+				const char* p;
+				char* d;
+			
+				if (!optMACinRsrc) {
+					/* try to create .rsrc/destination */
+					fprintf(stderr, _("Unable to copy MAC resource fork of %s because of %s does not support resource forks\n"),
+						source,
+						destination);
+						retValue = 1;
 					goto nomac;
 				}
-				dMAC = dcon;
-			}
-		}
-		if (!dMAC) {
-			const char* p;
-			char* d;
 			
-			if (!optMACinRsrc) {
-				/* try to create .rsrc/destination */
-				fprintf(stderr, _("Unable to copy MAC resource fork of %s because of %s does not support resource forks\n"),
-					source,
-					destination);
-					retValue = 1;
-				goto nomac;
-			}
-			
-			p = strrchr(destination, '/');
-			if (p)
-				p = p + 1;
-			else
-				p = destination;
-			memcpy(destinationB, destination, p - destination);
-			d = destinationB + (p - destination);
-			d = stpcpy(d, RSRCSUBDIR);
-			strcpy(d, p);
-			dFD = open(destinationB, O_CREAT | O_TRUNC | O_WRONLY, sourceMode);
-			if (dFD == -1) {
-				if (errno == ENOENT) {
-					*(d-1) = 0;
-					mkdir(destinationB, sourceMode | ((sourceMode & 0444) >> 2) | ((sourceMode & 0222) >> 1));
-					*(d-1) = '/';
-					dFD = open(destinationB, O_CREAT | O_TRUNC | O_WRONLY, sourceMode);
-				}
+				p = strrchr(destination, '/');
+				if (p)
+					p = p + 1;
+				else
+					p = destination;
+				memcpy(destinationB, destination, p - destination);
+				d = destinationB + (p - destination);
+				d = stpcpy(d, RSRCSUBDIR);
+				strcpy(d, p);
+				dFD = open(destinationB, O_CREAT | O_TRUNC | O_WRONLY, sourceMode);
 				if (dFD == -1) {
-					int err2 = errno;
+					if (errno == ENOENT) {
+						*(d-1) = 0;
+						mkdir(destinationB, sourceMode | ((sourceMode & 0444) >> 2) | ((sourceMode & 0222) >> 1));
+						*(d-1) = '/';
+						dFD = open(destinationB, O_CREAT | O_TRUNC | O_WRONLY, sourceMode);
+					}
+					if (dFD == -1) {
+						int err2 = errno;
 					
-					fprintf(stderr, _("Unable to copy MAC resource fork: %s: %s\n"),
-						destinationB,
-						strerror(err2));
-					retValue = 1;
-					goto nomac;
+						fprintf(stderr, _("Unable to copy MAC resource fork: %s: %s\n"),
+							destinationB,
+							strerror(err2));
+						retValue = 1;
+						goto nomac;
+					}
 				}
+				destinationRsrc = destinationB;
 			}
-			destinationRsrc = destinationB;
-		}
-		retValue = copyNWNW(_("NetWare copy (resource fork)"), 
-			    sFD, sMAC, sfhandle, sourceRsrc,
-			    dFD, dMAC, dfhandle, destinationRsrc);
+			retValue = copyNWNW(_("NetWare copy (resource fork)"), 
+				    sFD, sMAC, sfhandle, sourceRsrc,
+				    dFD, dMAC, dfhandle, destinationRsrc);
 nomac:;
-		if (dMAC)
-			ncp_close_file(dMAC, dfhandle);
-		if (dFD != -1)
-			close(dFD);
-		if (sMAC)
-			ncp_close_file(sMAC, sfhandle);
-		if (sFD != -1)
-			close(sFD);
+			if (dMAC)
+				ncp_close_file(dMAC, dfhandle);
+			if (dFD != -1)
+				close(dFD);
+			if (sMAC)
+				ncp_close_file(sMAC, sfhandle);
+			if (sFD != -1)
+				close(sFD);
+		}
+	} else {
+		retValue = 0;
 	}
 	if ((optTrustees) && (retValue == 0) && scon) {
 		u_int32_t iter = 0;
@@ -716,7 +742,7 @@ nomac:;
 				}
 			}
 			if (dptr) {
-				err = ncp_ns_trustee_add(dcon, NW_NS_DOS, SA_ALL,
+				err = ncp_ns_trustee_add(dcon, NW_NS_DOS, 0x0016,
 					1, dentry.volume, dentry.dirEnt, 
 					NULL, 0,
 					ts, dptr, 0xFFFF);
@@ -792,7 +818,21 @@ nomac:;
  *  file copy.
  */
 static int
-copyFiles(const char *source, const char *destination)
+selectFn(const struct dirent* entry) {
+	if (entry->d_name[0] == '.') {
+		if (entry->d_name[1] == 0)
+			return 0;
+		if (entry->d_name[1] == '.' && entry->d_name[2] == 0)
+			return 0;
+	}
+	return 1;
+}
+
+static char sourcePath[MAXPATHLEN * 2];
+static char destinationPath[MAXPATHLEN * 2];
+
+static int
+copyFiles(void)
 {
 	int oldUMask;
 	int retVal;
@@ -800,42 +840,137 @@ copyFiles(const char *source, const char *destination)
 	struct stat statBuf;
 #ifdef NCOPY_DEBUG
 	printf(_("Param Src   '%s'\n"
-	         "Param Dest  '%s'\n"), source, destination);
+	         "Param Dest  '%s'\n"), sourcePath, destinationPath);
 #endif
 
-	fdIn = open(source, O_RDONLY);
+	fdIn = open(sourcePath, O_RDONLY);
 	if (fdIn == -1)
 	{
-		fprintf(stderr, _("%s: Cannot open %s, %s\n"), ProgramName, source,
+		fprintf(stderr, _("%s: Cannot open %s, %s\n"), ProgramName, sourcePath,
 			strerror(errno));
 		return 1;
 	}
 	if (fstat(fdIn, &statBuf))
 	{
-		fprintf(stderr, _("%s: Cannot stat %s, %s\n"), ProgramName, source,
+		fprintf(stderr, _("%s: Cannot stat %s, %s\n"), ProgramName, sourcePath,
 			strerror(errno));
 		close(fdIn);
 		return 1;
 	}
 	if (S_ISDIR(statBuf.st_mode))
 	{
-		fprintf(stderr, _("%s: %s: omitting directory\n"), ProgramName, source);
-		close(fdIn);
-		return 0;	/* At this point, don't consider this a fatal error */
+		int ecode = 0;
+		
+		if (optRecursive) {
+			struct stat dstBuf;
+			struct dirent** entries;
+			int rent;
+			int i;
+			char *dpx;
+			char *spx;
+
+			if (!stat(destinationPath, &dstBuf)) {
+				if (!S_ISDIR(dstBuf.st_mode)) {
+					fprintf(stderr, _("%s: %s: %s\n"), ProgramName, destinationPath, _("not a directory"));
+					close(fdIn);
+					return 0;
+				}
+				oldUMask = umask(0);
+				if (dstBuf.st_mode != statBuf.st_mode) {
+					if (chmod(destinationPath, statBuf.st_mode & 0777)) {
+						fprintf(stderr, _("%s: Cannot chmod %s: %s\n"), ProgramName,
+							destinationPath, strerror(errno));
+					}
+				}
+			} else {
+				oldUMask = umask(0);
+				if (mkdir(destinationPath, statBuf.st_mode & 0777)) {
+					fprintf(stderr, _("%s: Cannot create %s: %s\n"), ProgramName, destinationPath,
+						strerror(errno));
+					umask(oldUMask);
+					close(fdIn);
+					return 1;
+				}
+			}
+			fdOut = open(destinationPath, O_RDONLY);
+			if (fdOut == -1)
+			{
+				fprintf(stderr, _("%s: Cannot create %s, %s\n"), ProgramName, destinationPath,
+					strerror(errno));
+				umask(oldUMask);
+				close(fdIn);
+				return 1;
+			}
+
+			retVal = netwareCopyFile(fdIn, fdOut, sourcePath, destinationPath, statBuf.st_mode, 0);
+			close(fdOut);
+			umask(oldUMask);
+			close(fdIn);
+			rent = scandir(sourcePath, &entries, selectFn, alphasort);
+			if (rent < 0) {
+				fprintf(stderr, _("%s: Cannot read %s: %s\n"), ProgramName, sourcePath, strerror(errno));
+				umask(oldUMask);
+				close(fdIn);
+				return 1;
+			}
+			dpx = destinationPath + strlen(destinationPath);
+			*dpx = '/';
+			spx = sourcePath + strlen(sourcePath);
+			*spx = '/';
+
+			for (i = 0; i < rent; i++) {
+				int tmp;
+				
+				strcpy(dpx + 1, entries[i]->d_name);
+				strcpy(spx + 1, entries[i]->d_name);
+				tmp = copyFiles();
+				if (tmp)
+					ecode = tmp;
+				if (tmp > 1)
+					break;
+				
+			}
+			*spx = 0;
+			*dpx = 0;
+			umask(oldUMask);
+			close(fdIn);
+		} else {
+			fprintf(stderr, _("%s: %s: omitting directory\n"), ProgramName, sourcePath);
+			close(fdIn);
+		}
+		return ecode;
 	}
 
 	oldUMask = umask(0);
-	fdOut = open(destination, O_CREAT | O_TRUNC | O_WRONLY, statBuf.st_mode);
+	if (optOnlyChanged) {
+		struct stat statBufOut;
+
+		if (!stat(destinationPath, &statBufOut)) {
+			if (statBuf.st_size == statBufOut.st_size &&
+			    statBuf.st_mtime == statBufOut.st_mtime) {
+			    	printf("%s and %s are same\n", sourcePath, destinationPath);
+			    	fdOut = open(destinationPath, O_RDONLY);
+			    	if (fdOut == -1) {
+			    		goto canopen;
+			    	}
+			    	retVal = netwareCopyFile(fdIn, fdOut, sourcePath, destinationPath, statBuf.st_mode, 0);
+			    	goto cpok;
+			}
+		}
+	}
+	fdOut = open(destinationPath, O_CREAT | O_TRUNC | O_WRONLY, statBuf.st_mode);
 	if (fdOut == -1)
 	{
-		fprintf(stderr, _("%s: Cannot create %s, %s\n"), ProgramName, destination,
+canopen:;
+		fprintf(stderr, _("%s: Cannot create %s, %s\n"), ProgramName, destinationPath,
 			strerror(errno));
 		umask(oldUMask);
 		close(fdIn);
 		return 1;
 	}
 
-	retVal = netwareCopyFile(fdIn, fdOut, source, destination, statBuf.st_mode);
+	retVal = netwareCopyFile(fdIn, fdOut, sourcePath, destinationPath, statBuf.st_mode, 1);
+cpok:;
 	close(fdOut);
 	umask(oldUMask);
 	close(fdIn);
@@ -861,9 +996,9 @@ copyFiles(const char *source, const char *destination)
  * Do we Stay in the loop?
  */
 static int
-copyRealPaths(const char *source, const char *destination)
+copyRealPaths(void)
 {
-	return copyFiles(source, destination);
+	return copyFiles();
 }
 
 /****************************************************************************
@@ -879,21 +1014,22 @@ handleFileArgs(int argc, char *const argv[])
 	int copyStatus;
 	int returnCode = 0;	/* default program exit code */
 	const char *baseNamePtr;
-	char destinationfile[MAXPATHLEN * 2];
 
 	destination = argv[argc - 1];	/* get LAST argument */
 	for (loop = 0; loop < (argc - 1); loop++)
 	{			/* all file arguments, but last */
-		strncpy(destinationfile, destination, MAXPATHLEN);
-		destinationfile[MAXPATHLEN] = 0;
+		strncpy(destinationPath, destination, MAXPATHLEN);
+		destinationPath[MAXPATHLEN] = 0;
 		if ((argc > 2) || (!notDir(argv[argc - 1])))
 		{		/* destination is a dir */
-			if (*destinationfile != '/' || *(destinationfile + 1))
-				strcat(destinationfile, "/");
+			if (*destinationPath != '/' || *(destinationPath + 1))
+				strcat(destinationPath, "/");
 			baseNamePtr = myBaseName(argv[loop]);	/* get the file name */
-			strcat(destinationfile, baseNamePtr);	/* add it on end of directory */
+			strcat(destinationPath, baseNamePtr);	/* add it on end of directory */
 		}
-		copyStatus = copyRealPaths(argv[loop], destinationfile);	/* do the copy */
+		strncpy(sourcePath, argv[loop], MAXPATHLEN);
+		sourcePath[MAXPATHLEN] = 0;
+		copyStatus = copyRealPaths();	/* do the copy */
 		if (copyStatus > 1)
 			return copyStatus;	/* fatal failure? bye */
 		if (copyStatus == 1)
@@ -1032,8 +1168,8 @@ main(int argc, char *const argv[])
 		ProgramName = argv[0];
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(NCPFS_PACKAGE, LOCALEDIR);
+	textdomain(NCPFS_PACKAGE);
 	
 	if (handleOptions(argc, argv))
 	{			/* bad option, missing option parameter */
