@@ -24,11 +24,15 @@
 
 	1.00  2000, January 15		Petr Vandrovec <vandrove@vc.cvut.cz>
 		NDS queues support.
+
+	1.01  2000, September 11	Bruno Browning <bruno@lss.wirc.edu>
+		Added '-s' option, changed '-b' to '-B' to get it to work.
+
+	1.02  2000, Feburary 18		Thomas Hood <jdthood@ubishops.ca>
+		Use STRCPY_SAFE when possible, clarify help and manpage.
 		
  */
 
-#include "config.h"
-
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
@@ -38,61 +42,25 @@
 #include <ctype.h>
 
 #include <ncp/nwnet.h>
+#include "dsqueue.h"
 
-#include <libintl.h>
+#include "private/libintl.h"
 #define _(X) gettext(X)
+#define STRCPY_SAFE(d,s) { strncpy((d),(s),sizeof(d)-1) ; (d)[sizeof(d)-1] = '\0' ; }
 
 static char *progname;
 static void usage(void);
 static void help(void);
 
-static NWDSCCODE try_nds_queue(NWCONN_HANDLE* conn, char* name, NWObjectID* id) {
-	NWDSContextHandle ctx;
-	NWDSCCODE err;
-	nuint32 flags;
-	char host[MAX_DN_CHARS+1];
-	NWCONN_HANDLE nconn;
-
-	err = NWDSCreateContextHandle(&ctx);
-	if (err)
-		goto quit;
-	err = NWDSAddConnection(ctx, *conn);
-	if (err)
-		goto freectx;
-	err = NWDSGetContext(ctx, DCK_FLAGS, &flags);
-	if (err)
-		goto freectx;
-	flags &= ~DCV_CANONICALIZE_NAMES;
-	err = NWDSSetContext(ctx, DCK_FLAGS, &flags);
-	if (err)
-		goto freectx;
-	/* if object does not have Host Server property, assume damaged queue...
-	   We use provided server in that case... */
-	err = NWDSGetObjectHostServerAddress(ctx, name, host, NULL);
-	if (err)
-		goto simple;
-	/* if we cannot connect to target server, ignore it... */
-	err = NWDSOpenConnToNDSServer(ctx, host, &nconn);
-	if (err)
-		goto simple;
-	/* swap connections to get to right server */
-	NWCCCloseConn(*conn);
-	*conn = nconn;
-simple:;		
-	err = NWDSMapNameToID(ctx, *conn, name, id);
-freectx:;
-	NWDSFreeContext(ctx);
-quit:;
-	return err;
-}
-
 int
 main(int argc, char *argv[])
 {
-	static char* default_queue = "*";
+	static char default_queue[] = "*";
 	char *queue = default_queue;
 
 	int opt;
+	
+	int requestBanner, supressBanner;
 
 	struct ncp_bindery_object q;
 	struct queue_job j;
@@ -102,15 +70,15 @@ main(int argc, char *argv[])
 
 	char buf[8192];
 
-	char *file_name;
+	const char *file_name;
 	int file;
 	long err;
 
 	struct ncp_conn *conn;
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(NCPFS_PACKAGE, LOCALEDIR);
+	textdomain(NCPFS_PACKAGE);
 	
 	progname = argv[0];
 
@@ -139,7 +107,7 @@ main(int argc, char *argv[])
 	/* at once */
 	memset(&(j.j.TargetExecTime), 0xff, sizeof(j.j.TargetExecTime));
 	j.j.JobType = htons(0);
-	strcpy(j.j.JobTextDescription, _("No Description"));
+	STRCPY_SAFE(j.j.JobTextDescription, _("No Description"));
 
 	pj.Version = 0;
 	pj.TabSize = 8;
@@ -148,8 +116,11 @@ main(int argc, char *argv[])
 	pj.Lines = htons(66);
 	pj.Rows = htons(80);
 	strcpy(pj.FnameHeader, "stdin");
+	
+	requestBanner = 0;
+	supressBanner = 0;
 
-	while ((opt = getopt(argc, argv, "h?q:d:p:b:f:l:r:c:t:F:TN")) != EOF)
+	while ((opt = getopt(argc, argv, "h?q:d:p:B:sf:l:r:c:t:F:TN")) != EOF)
 	{
 		switch (opt)
 		{
@@ -160,39 +131,22 @@ main(int argc, char *argv[])
 			exit(1);
 		case 'p':
 			/* Path */
-			pj.CtrlFlags |= PRINT_BANNER;
-			if (strlen(optarg) >= sizeof(pj.Path))
-			{
-				strncpy(pj.Path, optarg,
-					sizeof(pj.Path));
-			} else
-			{
-				strcpy(pj.Path, optarg);
-			}
+			requestBanner = 1;
+			STRCPY_SAFE(pj.Path, optarg);
 			break;
-		case 'b':
+		case 'B':
 			/* Banner Name */
-			pj.CtrlFlags |= PRINT_BANNER;
-			if (strlen(optarg) >= sizeof(pj.BannerName))
-			{
-				strncpy(pj.BannerName, optarg,
-					sizeof(pj.BannerName));
-			} else
-			{
-				strcpy(pj.BannerName, optarg);
-			}
+			requestBanner = 1;
+			STRCPY_SAFE(pj.BannerName, optarg);
+			break;
+		case 's':
+			/* Supress Banner Page */
+			supressBanner = 1;
 			break;
 		case 'f':
 			/* File Name in Banner */
-			pj.CtrlFlags |= PRINT_BANNER;
-			if (strlen(optarg) >= sizeof(pj.FnameBanner))
-			{
-				strncpy(pj.FnameBanner, optarg,
-					sizeof(pj.FnameBanner));
-			} else
-			{
-				strcpy(pj.FnameBanner, optarg);
-			}
+			requestBanner = 1;
+			STRCPY_SAFE(pj.FnameBanner, optarg);
 			break;
 		case 'l':
 			/* lines, default: 66 */
@@ -258,26 +212,12 @@ main(int argc, char *argv[])
 			break;
 		case 'q':
 			/* Queue name to print on, default: '*' */
-			if (strlen(optarg) >= NCP_BINDERY_NAME_LEN)
-			{
-				printf(_("queue name too long: %s\n"),
-				       optarg);
-				ncp_close(conn);
-				exit(1);
-			}
 			queue = optarg;
 			break;
 		case 'd':
 			/* Job Description */
-			pj.CtrlFlags |= PRINT_BANNER;
-			if (strlen(optarg) >= sizeof(j.j.JobTextDescription))
-			{
-				strncpy(j.j.JobTextDescription, optarg,
-					sizeof(j.j.JobTextDescription));
-			} else
-			{
-				strcpy(j.j.JobTextDescription, optarg);
-			}
+			requestBanner = 1;
+			STRCPY_SAFE(j.j.JobTextDescription, optarg);
 			break;
 
 		default:
@@ -286,20 +226,25 @@ main(int argc, char *argv[])
 			exit(1);
 		}
 	}
-
-	if (optind != argc - 1)
+	
+	if (requestBanner && !supressBanner)
 	{
+		pj.CtrlFlags |= PRINT_BANNER;
+	}
+
+	if (optind < argc - 1) {
 		usage();
 		ncp_close(conn);
 		exit(1);
+	} else if (optind == argc - 1) {
+		file_name = argv[optind];
+	} else {
+		file_name = "-";
 	}
-	file_name = argv[optind];
 
-	if (strcmp(file_name, "-") == 0)
-	{
+	if (strcmp(file_name, "-") == 0) {
 		file = 0;	/* stdin */
-	} else
-	{
+	} else {
 		file = open(file_name, O_RDONLY, 0);
 		if (file < 0)
 		{
@@ -307,26 +252,9 @@ main(int argc, char *argv[])
 			ncp_close(conn);
 			exit(1);
 		}
-		if (strlen(file_name) >= sizeof(pj.FnameHeader))
-		{
-			strncpy(pj.FnameHeader, file_name,
-				sizeof(pj.FnameHeader));
-		} else
-		{
-			strcpy(pj.FnameHeader, file_name);
-		}
-
+		STRCPY_SAFE(pj.FnameHeader, file_name);
 		if (strlen(pj.FnameBanner) == 0)
-		{
-			if (strlen(file_name) >= sizeof(pj.FnameBanner))
-			{
-				strncpy(pj.FnameBanner, file_name,
-					sizeof(pj.FnameBanner));
-			} else
-			{
-				strcpy(pj.FnameBanner, file_name);
-			}
-		}
+			STRCPY_SAFE(pj.FnameBanner, file_name);
 	}
 
 	memcpy(j.j.ClientRecordArea, &pj, sizeof(pj));
@@ -399,9 +327,10 @@ help(void)
 	       "-C             Don't convert password to uppercase\n"
 	       "-q queue name  Name of the printing queue to use\n"
 	       "-d job desc    Job description\n"
-	       "-p path name   Path name to appear on banner\n"
-	       "-b bannername  Banner name (up to 12 chars)\n"
-	       "-f filename    Filename to appear on banner\n"
+	       "-p pathname    Pathname to appear on banner (up to 79 chars)\n"
+	       "-B username    Username to appear on banner (up to 12 chars)\n"
+	       "-f filename    Filename to appear on banner (up to 12 chars)\n"
+	       "-s             Supress banner page\n"
 	       "-l lines       Number of lines per page\n"
 	       "-r rows        Number of rows per page\n"
 	       "-t tab         Number of spaces per tab\n"
