@@ -34,7 +34,7 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <libintl.h>
+#include "private/libintl.h"
 #define _(X) gettext(X)
 
 static char *progname;
@@ -96,8 +96,8 @@ main(int argc, char *argv[])
 	int verbose = 0;
 		
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(NCPFS_PACKAGE, LOCALEDIR);
+	textdomain(NCPFS_PACKAGE);
 	
 	progname = argv[0];
 
@@ -146,6 +146,7 @@ main(int argc, char *argv[])
 			ncp_close(conn);
 			continue;
 		}
+#ifdef NDS_SUPPORT
 		{
 			u_int16_t ver;
 		
@@ -153,12 +154,15 @@ main(int argc, char *argv[])
 			if (!err && (ver >= 0x0400)) {
 				errctx = NWDSCreateContextHandle(&ctx);
 				if (!errctx) {
-					static const u_int32_t flags = DCV_XLATE_STRINGS;
+					u_int32_t flags;
+					NWDSAddConnection(ctx, conn);
+					NWDSGetContext(ctx, DCK_FLAGS, &flags);
+					flags |= DCV_XLATE_STRINGS;
 					NWDSSetContext(ctx, DCK_FLAGS, &flags);
 				}
 			}
 		}
-		
+#endif		
 		iter = 0;
 		while (1) {
 			unsigned int i;
@@ -180,10 +184,12 @@ main(int argc, char *argv[])
 				char name[1024];
 				u_int16_t type;
 			
-				if (errctx)
-					err = NWGetObjectName(conn, tinfo[i].objectID, name, &type);
-				else
+#ifdef NDS_SUPPORT
+				if (!errctx)
 					err = NWDSMapIDToName(ctx, conn, tinfo[i].objectID, name);
+				else
+#endif
+					err = NWGetObjectName(conn, tinfo[i].objectID, name, &type);
 				if (err)
 					sprintf(name, "<#%08X>", tinfo[i].objectID);
 				printf("%s %s\n", maskToText(tinfo[i].objectRights), name);
@@ -193,8 +199,10 @@ main(int argc, char *argv[])
 			fprintf(stderr, _("Trustee scan failed, %s\n"), strnwerror(err));
 		result = 0;
 		ncp_close(conn);
+#ifdef NDS_SUPPORT
 		if (!errctx)
 			NWDSFreeContext(ctx);
+#endif
 		errctx = 1;
 	}
 finished:
