@@ -25,9 +25,11 @@
 
 	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
 		Added license.
- */
+		
+	1.01  2001, February 18		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added support for NDS queues.
 
-#include "config.h"
+ */
 
 #include <stdio.h>
 #include <unistd.h>
@@ -35,7 +37,9 @@
 #include <stdlib.h>
 #include <ncp/nwcalls.h>
 
-#include <libintl.h>
+#include "dsqueue.h"
+
+#include "private/libintl.h"
 #define _(X) gettext(X)
 
 int
@@ -47,8 +51,8 @@ main(int argc, char **argv)
 	int i;
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(NCPFS_PACKAGE, LOCALEDIR);
+	textdomain(NCPFS_PACKAGE);
 	
 	if ((conn = ncp_initialize(&argc, argv, 1, &err)) == NULL)
 	{
@@ -66,20 +70,24 @@ main(int argc, char **argv)
 	if (ncp_get_bindery_object_id(conn, NCP_BINDERY_PQUEUE, 
                                       argv[1], &q) != 0)
 	{
-		char server[NW_MAX_SERVER_NAME_LEN];
-		const char* sptr = server;
-		NWCCODE err;
+		NWObjectID id;
+		
+		if (try_nds_queue(&conn, argv[1], &id)) {
+			char server[NW_MAX_SERVER_NAME_LEN];
+			const char* sptr = server;
+			NWCCODE err2;
 
-		err = NWCCGetConnInfo(conn, NWCC_INFO_SERVER_NAME,
-			sizeof(server), server);
-		if (err)
-			sptr = "?";
-
-                printf(_("Queue \"%s\" on server %s not found.\n"),
-                       argv[1], sptr);
-                ncp_close(conn);
-                exit(1);
+			err2 = NWCCGetConnInfo(conn, NWCC_INFO_SERVER_NAME,
+				sizeof(server), server);
+			if (err2)
+				sptr = "?";
 
+	                printf(_("Queue \"%s\" on server %s not found.\n"),
+                	       argv[1], sptr);
+        	        ncp_close(conn);
+	                exit(1);
+		}
+		q.object_id = id;
 	}
 
 	for (i=2; i<argc; i++) {
@@ -90,19 +98,18 @@ main(int argc, char **argv)
 		if (*end) {
 			fprintf(stderr, _("Cannot parse \"%s\" - jobID must be hexadecimal number\n"), argv[i]);
 		} else {
-			int err;
+			int err2;
 
-			err = NWRemoveJobFromQueue2(conn, q.object_id, jobID);
-			if (err == 0x89D5) {
+			err2 = NWRemoveJobFromQueue2(conn, q.object_id, jobID);
+			if (err2 == 0x89D5) {
 				fprintf(stderr, _("Job %08X does not exist\n"), jobID);
-			} else if (err == 0x89D6) {
+			} else if (err2 == 0x89D6) {
 				fprintf(stderr, _("You have not rights to cancel job %08X\n"), jobID);
-			} else if (err) {
-				fprintf(stderr, _("Cannot cancel job %08X: %s\n"), jobID, strnwerror(err));
+			} else if (err2) {
+				fprintf(stderr, _("Cannot cancel job %08X: %s\n"), jobID, strnwerror(err2));
 			}
 		}
 	}
 	ncp_close(conn);
 	return 0;
 }
-
