@@ -24,24 +24,28 @@
 
 	1.00  1999, November 20		Petr Vandrovec <vandrove@vc.cvut.cz>
 		Added license.
+
+	1.01  2001, February 18		Petr Vandrovec <vandrove@vc.cvut.cz>
+		Added support for NDS queues.
   
  */
 
-#include "config.h"
-
 #include <stdio.h>
 #include <unistd.h>
 #include <ctype.h>
 #include <stdlib.h>
 #include <time.h>
+#include <string.h>
 #include <ncp/nwcalls.h>
 
-#include <libintl.h>
+#include "dsqueue.h"
+
+#include "private/libintl.h"
 #define _(X) gettext(X)
 #define N_(X) (X)
 
 /* move this to library ? */
-int
+static int
 ncp_time_to_tm(struct tm* out, u_int8_t* netwareTime)
 {
 	struct tm tmp;
@@ -62,12 +66,12 @@ ncp_time_to_tm(struct tm* out, u_int8_t* netwareTime)
 	return 0;
 }
 
-int
+static int
 ncp_cmp_time(struct tm* tm1, struct tm* tm2)
 {
 #undef XTST
-#define XTST(Y) if (tm1->tm_##Y## != tm2->tm_##Y##) { \
-			return (tm1->tm_##Y## > tm2->tm_##Y##) ? 1 : -1; \
+#define XTST(Y) if (tm1->tm_##Y != tm2->tm_##Y) { \
+			return (tm1->tm_##Y > tm2->tm_##Y) ? 1 : -1; \
 		}
 	XTST(year);
 	XTST(mon);
@@ -79,20 +83,41 @@ ncp_cmp_time(struct tm* tm1, struct tm* tm2)
 	return 0;
 }
 
+static void get_object_name(NWCONN_HANDLE conn, NWObjectID objID, char* buff) {
+	NWCCODE err;
+
+	err = NWGetObjectName(conn, objID, buff, NULL);
+	if (err) {
+		NWDSContextHandle ctx;
+
+		err = NWDSCreateContextHandle(&ctx);
+		if (!err) {
+			err = NWDSAddConnection(ctx, conn);
+			if (!err) {
+				err = NWDSMapIDToName(ctx, conn, objID, buff);
+			}
+			NWDSFreeContext(ctx);
+		}
+	}
+	if (err) {
+		sprintf(buff, "<%s>", strnwerror(err));
+	}
+}
+
 int
 main(int argc, char **argv)
 {
 	struct ncp_conn *conn;
-	struct ncp_bindery_object q,u;
-        unsigned long maxqlen=~0;
+	struct ncp_bindery_object q;
+	unsigned long maxqlen=~0;
 	long err;
 
-        u_int32_t qlen, idl1,idl2, job_id;
-        struct nw_queue_job_entry j;
+	u_int32_t qlen, idl1,idl2, job_id;
+	struct nw_queue_job_entry j;
 
 	setlocale(LC_ALL, "");
-	bindtextdomain(PACKAGE, LOCALEDIR);
-	textdomain(PACKAGE);
+	bindtextdomain(NCPFS_PACKAGE, LOCALEDIR);
+	textdomain(NCPFS_PACKAGE);
 	
 	if ((conn = ncp_initialize(&argc, argv, 1, &err)) == NULL)
 	{
@@ -100,8 +125,7 @@ main(int argc, char **argv)
 		return 1;
 	}
 
-        if (argc == 3)
-        {
+	if (argc == 3) {
                 char *end;
 
                 maxqlen=strtoul(argv[2], &end, 10);
@@ -119,34 +143,40 @@ main(int argc, char **argv)
 	if (ncp_get_bindery_object_id(conn, NCP_BINDERY_PQUEUE, 
                                       argv[1], &q) != 0)
 	{
-		char server[NW_MAX_SERVER_NAME_LEN];
-		const char* sptr = server;
-		NWCCODE err;
-
-		err = NWCCGetConnInfo(conn, NWCC_INFO_SERVER_NAME, 
-			sizeof(server), server);
-		if (err)
-			sptr = "?";
-               	printf(_("Queue \"%s\" on server %s not found.\n"),
-			argv[1], server);
-                ncp_close(conn);
-                exit(1);
+		NWObjectID id;
+		
+		if (try_nds_queue(&conn, argv[1], &id)) {
+			char server[NW_MAX_SERVER_NAME_LEN];
+			const char* sptr = server;
+			NWCCODE err2;
 
+			err2 = NWCCGetConnInfo(conn, NWCC_INFO_SERVER_NAME, 
+				sizeof(server), server);
+			if (err2)
+				sptr = "?";
+	               	printf(_("Queue \"%s\" on server %s not found.\n"),
+				argv[1], server);
+                	ncp_close(conn);
+	                exit(1);
+		}
+		q.object_id = id;
 	}
 
 	if (isatty(1))
 	{
 		char server[NW_MAX_SERVER_NAME_LEN];
+		char qname[MAX_DN_BYTES];
 		const char* sptr = server;
-		NWCCODE err;
+		NWCCODE err2;
 
-		err = NWCCGetConnInfo(conn, NWCC_INFO_SERVER_NAME,
+		err2 = NWCCGetConnInfo(conn, NWCC_INFO_SERVER_NAME,
 			sizeof(server), server);
-		if (err)
+		if (err2)
 			sptr = "?";
 
+		get_object_name(conn, q.object_id, qname);
 		printf(_("\nServer: %s\tQueue: %s\tQueue ID: %8.8X\n"), sptr, 
-			q.object_name, q.object_id);
+			qname, q.object_id);
                 printf(_(" %5s  %-12s  %-32s  %-7s  %-4s  %-8s\n"
 		       "-----------------------------------------------"
 		       "--------------------------------\n"),
@@ -183,19 +213,9 @@ main(int argc, char **argv)
         while (maxqlen-- && job_id && (ncp_get_queue_job_info(conn, q.object_id, job_id, &j) == 0))
 	{
 		const char* jst;
+		char user[MAX_DN_BYTES];
 
-                char user[50];
-                if ((ncp_get_bindery_object_name
-                     (conn, ntohl(j.ClientObjectID), &u))
-                    == 0)
-                {
-                        memcpy(user,u.object_name,48);
-			user[48]=0;
-                } 
-                else
-                {
-                        sprintf(user,_("<Unknown>"));
-                }
+		get_object_name(conn, ntohl(j.ClientObjectID), user);
 
                 j.JobFileName[j.FileNameLen]=0;
 
@@ -231,4 +251,3 @@ main(int argc, char **argv)
 	ncp_close(conn);
 	return 0;
 }
-
